#include "FGDependencyContext.h"

#include "CharUtils.h"

#include <sstream>
#include <assert.h>

using namespace std;

namespace egstra {
  void fg_dependency_context::phi(const sentence* x, list<string> F[],
				  const int word_limit,
				  const bool use_pos) {
    if(!use_pos) { return; }
    const int N = x->size();
    /* precompute the array of tag strings with padding */
    string* ctags = new string[N + 2];
    ctags[0]     = "+**";                   /* pre-sentence marker */
    ctags[N + 1] = "+**";                  /* post-sentence marker */
    ++ctags;
    ostringstream os;
    for(int i = 0; i < N; ++i) {
      os.str("");
      os << "+" << x->get_token(i).coarse_tag();
      ctags[i] = os.str();
    }

    for(int s = 0; s < N; ++s) {                    /* start index */
      {                                         /* root dependency */
	list<string>& f = F[s + N*s];
	const string s_r("root:ctxt" + ctags[s]);
	f.push_back(s_r + "[-+]" + ctags[s - 1] + ctags[s + 1]);
	f.push_back(s_r + "[-?]" + ctags[s - 1]);
	f.push_back(s_r + "[?+]" + ctags[s + 1]);
      }

#define addf1(desc, str)			\
      {						\
	const string rest(desc + str);		\
	f_se.push_back(s_se + rest);		\
	f_es.push_back(s_es + rest);		\
      }
#define addf2(desc, str1, str2)			\
      {						\
	const string rest(desc + str1 + str2);	\
	f_se.push_back(s_se + rest);		\
	f_es.push_back(s_es + rest);		\
      }

      if(s + 1 < N) {                       /* adjacent dependency */
	list<string>& f_se = F[s + N*(s + 1)];  /* feats for s-->e */
	list<string>& f_es = F[s + 1 + N*s];    /* feats for e-->s */
	const string dep(ctags[s] + ctags[s + 1]);
	const string s_se("a:0+ctxt" + dep);
	const string s_es("a:1+ctxt" + dep);
	addf2("[-+]", ctags[s - 1], ctags[s + 2]);
	addf1("[-?]", ctags[s - 1]);
	addf1("[?+]", ctags[s + 2]);
      }
      for(int e = s + 2; e < N; ++e) {                /* end index */
	list<string>& f_se = F[s + N*e];        /* feats for s-->e */
	list<string>& f_es = F[e + N*s];        /* feats for e-->s */
	const string dep(ctags[s] + ctags[e]);
	const string s_se("d:0+ctxt" + dep);
	const string s_es("d:1+ctxt" + dep);
	addf2("[++]", ctags[s + 1], ctags[e + 1]);
	addf2("[+-]", ctags[s + 1], ctags[e - 1]);
	addf2("[-+]", ctags[s - 1], ctags[e + 1]);
	addf2("[--]", ctags[s - 1], ctags[e - 1]);
	addf1("[+?]", ctags[s + 1]);
	addf1("[-?]", ctags[s - 1]);
	addf1("[?+]", ctags[e + 1]);
	addf1("[?-]", ctags[e - 1]);
      }
    }

    /* recover the original pointer for deletion */
    --ctags;
    delete [] ctags;
  }
}
