#include "FGDependencyDistance.h"

#include <sstream>
#include <assert.h>

using namespace std;

/* Threshold for the token distance within a deependency */
#define EGSTRA_TOKEN_DISTANCE_THRESHOLD 41

namespace egstra {
  void fg_dependency_distance::phi(const sentence* const x, list<string> F[],
				   const int word_limit,
				   const bool use_pos) {
    const int N = x->size();
    ostringstream os;
    for(int h = 0; h < N; ++h) {
      for(int m = 0; m < N; ++m) {
	/* skip the root dependency */
	if(h == m) { continue; }

	const int dep_idx = h + N*m;
	list<string>& f = F[dep_idx];

	/* the start of the dependency in the sentence */
	const int dist = (m < h ? h - m : m - h);
	/* indicator for the direction of the dependency */
	const int modBefore = (m < h ? 1 : 0);

	/* binned features */
#define FG(prefix, len) \
	if(dist > len) { \
	  os.str(""); \
	  os << prefix << len; \
	  f.push_back(os.str()); \
	}

#define FGall(prefix) \
        FG(prefix, 2); \
        FG(prefix, 5); \
        FG(prefix, 10); \
        FG(prefix, 20); \
        FG(prefix, 30); \
        FG(prefix, 40);

	os.str("");
	os << "pathlenge:" << modBefore << "+";
	const string plain_pfx = os.str();
	FGall(plain_pfx);

	if(use_pos) {
	  /* the token to be inspected */
	  const token& ht = x->get_token(h);
	  const token& mt = x->get_token(m);

	  os.str("");
	  os << "pathlenge[h]:" << modBefore << "+"
	     << ht.coarse_tag() << "+";
	  const string head_pfx = os.str();
	  os.str("");
	  os << "pathlenge[m]:" << modBefore << "+"
	     << mt.coarse_tag() << "+";
	  const string mod_pfx = os.str();
	  os.str("");
	  os << "pathlenge[h,m]:" << modBefore << "+"
	     << ht.coarse_tag() << "+"
	     << mt.coarse_tag() << "+";
	  const string dep_pfx = os.str();

	  FGall(head_pfx);
	  FGall(mod_pfx);
	  FGall(dep_pfx);
	}


        /* feature for the actual length of the dependency */
	const int plen = (dist > EGSTRA_TOKEN_DISTANCE_THRESHOLD
			  ? EGSTRA_TOKEN_DISTANCE_THRESHOLD : dist);
	os.str("");
	os << "pathlen:" << modBefore << "+" << plen;
	f.push_back(os.str());
      }
    }
  }
}

#undef FG
#undef FGall
