/* DualVector : a class that holds one parameter for each (i,r) pair
                in a dataset.

   Author : Terry Koo
            maestro@csail.mit.edu */
#ifndef EGSTRA_DUAL_VECTOR_H
#define EGSTRA_DUAL_VECTOR_H

#include "Pattern.h"

#include <string>

namespace egstra {
  class dual_vector {
  private:
    const int _N;
    const std::string _cachedir;
    const std::string _prefix;
    const double** _theta;
    const bool _gzipped;
    const bool _usedisk;
    /* cached filenames to eliminate unnecessary sprintf-ing */
    char* _rd_fname;
    char* _wr_fname;
    int _rd_offset;
    int _wr_offset;

  public:
    /* create a set of duals for a dataset of N examples.  if dir is
       the empty string, then the duals will be cached in memory.  if
       dir is non-empty, then it is taken to be the pathname of a
       directory in which to store the duals on disk.  the prefix is
       an optional argument that will be prepended to each file (it
       may be useful for uniqueness if one will be storing many sets
       of dual vectors in the same directory) */
    dual_vector(const int N, const std::string& dir, const bool gzipped,
		const std::string& prefix = "d");
    ~dual_vector();

    /* retrieve the duals for the given pattern.  NB: the dual vectors
       will *not* be initialized, and thus you must set() any value
       before retrieving it with get() */
    const double* get(const pattern* const x);
    /* store a set of duals for the given pattern.  NB: the theta
       vector *must* have been allocated with new double[], not
       malloc().  also, after calling set() on some theta vector, the
       vector may be deallocated and it is not safe to refer to this
       theta after set() */
    void set(const pattern* const x,
	     const double* const theta);
    /* discard a dual vector.  call this on vectors that you have
       retrieved with get(), but which you no longer need, and which
       you do not want to save with set(). */
    void discard(const double* theta);

  };
}

#endif /* EGSTRA_DUAL_VECTOR_H */
