/**
 * Contains some string manipulation functions
 * @file CharUtils.cc
 * @author Mihai Surdeanu
 */

#include <list>
#include <stdlib.h>
#include <ctype.h>

#include "CharUtils.h"
#include "CppAssert.h"

using namespace std;

namespace egstra {
  bool startsWith(const std::string & big,
		  const std::string & small)
  {
    if(small.size() > big.size()) return false;

    for(size_t i = 0; i < small.size(); i ++){
      if(small[i] != big[i]){
	return false;
      }
    }

    return true;
  }

  bool endsWith(const std::string & big,
		const std::string & small)
  {
    size_t bigSize = big.size();
    size_t smallSize = small.size();

    if(smallSize > bigSize) return false;

    for(size_t i = 0; i < smallSize; i ++){
      if(small[i] != big[bigSize - smallSize + i]){
	return false;
      }
    }

    return true;
  }

  int toInteger(const std::string & s)
  {
    return strtol(s.c_str(), NULL, 10);
  }

  void toString(const int i, std::string& s) {
    std::ostringstream os;
    os << i;
    s = os.str();
  }


  void simpleTokenize(const std::string & input,
		      std::vector<std::string> & output,
		      const std::string & separators)
  {
    for(int start = input.find_first_not_of(separators);
	start < (int) input.size() && start >= 0;
	start = input.find_first_not_of(separators, start)){
      int end = input.find_first_of(separators, start);
      if(end < 0) end = input.size();
      output.push_back(input.substr(start, end - start));
      start = end;
    }
  }

  std::string toUpper(const std::string & s)
  {
    std::string o = s;

    for(int i = 0; i < (int) o.size(); i ++){
      o[i] = toupper(o[i]);
    }

    return o;
  }

  std::string toLower(const std::string & s)
  {
    std::string o = s;

    for(int i = 0; i < (int) o.size(); i ++){
      o[i] = tolower(o[i]);
    }

    return o;
  }

  std::string stripString(const std::string & s,
			  int left,
			  int right)
  {
    return s.substr(left, s.size() - left - right);
  }

  bool emptyLine(const std::string & input)
  {
    for(size_t i = 0; i < input.size(); i ++)
      if(! isblank(input[i])) return false;
    return true;
  }


  /*
    int main(int argc, char ** argv)
    {
    String p = argv[1];
    vector<String> exp;
    expandNumber(p, exp);
    for(int i = 0; (int) i < exp.size(); i ++){
    cout << "\"" << exp[i] << "\"" << " ";
    }
    cout << endl;
    }
  */
}
