/* FGen : abstract class representing a feature mapping

   Create a subclass with the virtual functions defined.

   Author : Terry Koo
            maestro@csail.mit.edu */

#ifndef EGSTRA_FGEN_H
#define EGSTRA_FGEN_H

#include "Pattern.h"
#include "FVec.h"

namespace egstra {
  class fgen {
  protected:
    const char* _name;
    int _dim;             /* feature dimensionality */

  public:
    fgen(const char* name)
      : _name(name), _dim(-1) {}
    virtual ~fgen() {}

    /* accessors */
    int dim() const          { return _dim; }
    const char* name() const { return _name; }

    /* handle the command line */
    virtual void usage(const char* const mesg = "") = 0;
    virtual void process_options() = 0;

    /* load and save feature mappings */
    virtual void load_dictionaries();
    virtual void save_dictionaries();

    /* return an array of feature vectors that correspond to the
       various parts.  specifically, phi(x)[r] should give the
       features associated with part r. */
    virtual const struct fvec* phi(const pattern* const x) = 0;

    /* return a single feature vector corresponding to the part at
       index r. */
    virtual const struct fvec* phi(const pattern* const x, const int r);

    /* return the coordinates to the actual features of the part, that
       is a pointer to the features (fv_index) and the offset from
       feature indices to actual parameters of the model */
    virtual void part_to_fvec_coordinates(const pattern* const x, const int r, int *fv_index, int *offset);

    /* these functions will be called when the features are done being
       used.  either of the two functions below will be called
       depending on which of the above functions was used to generate
       the feature vectors */
    virtual void discard(const struct fvec* const F,
			 const pattern* const x) = 0;
    virtual void discard(const struct fvec* const F,
			 const pattern* const x, const int r);

  };
}

#endif /* EGSTRA_FGEN_H */
