#include "OracleDecoder.h"

#include "Options.h"
#include "OptUtils.h"
#include "Timer.h"

#include <stdlib.h>

#include <iostream>
#include <string>

using namespace std;

namespace egstra {
  void oracle_decoder::usage(const char* const mesg) {
    cerr << "Oracle decoding options:" << endl;
    cerr << " --random           : use random scores" << endl;
    cerr << " --output=<string>  : output filename" << endl;
    cerr << endl;
    cerr << mesg << endl;
  }

  void oracle_decoder::process_options() {
    int tmp;
    _random = false;
    if(options::get("random", tmp)) {
      _random = true;
    }
    _output = "";
    options::get("output", _output);
  }

  void oracle_decoder::train(dataset& trdata, dataset& valdata,
			     const int T) {

    //optutils::classify_val_set(w, trdata, *_fg, *_M,

    timer_start();

    /* open output file */
    char buf[64];
    string maxfilename;
    if (_output != "") {
      maxfilename = _output;
    }
    else if (_random) {
      sprintf(buf, "random-labels");
      maxfilename = _dir + "/" + buf;
    }
    else {
      sprintf(buf, "oracle-labels");
      maxfilename = _dir + "/" + buf;
    }

    FILE* maxout = fopen(maxfilename.c_str(), "w");
    assert(maxout != NULL);


    const int N = trdata.size();
    int cnt = 0;
    dataset::const_iterator it = trdata.begin();
    const dataset::const_iterator it_end = trdata.end();
    for(; it != it_end; ++it) {
      const example* ex = *it;
      const pattern* const x = ex->x();

      /* construct matrix of scores */
      const int R = x->nparts();
      double* const S = (double*)malloc(R*sizeof(double));

      if (_random) {      
	double *s = S;
	for (int r=0; r<R; ++r, ++s) {	  
	  *s = rand();
	}
      }      
      else {
	double *s = S;
	for (int r=0; r<R; ++r, ++s) {	  
	  *s = 0.0;
	}
	const label& y = ex->y();
	label::const_iterator it = y.begin();
	const label::const_iterator it_end = y.end();
	for ( ; it != it_end; ++it) {
	  S[*it] = 1.0;
	}
      }

      /* find the argmax (viterbi) label */
      label ymax;
      _M->argmax(x, S, ymax);
      _M->fwrite(maxout, x, ymax);
      free(S);

      ++cnt;
      ticker(cnt, N);
    }
    ticker(N, N);
    fclose(maxout);
    timer_stop();

    cerr << " done [" << timestr(timediff) << "]" << endl;

  }
}
