/* Trainer : an abstract class representing a parameter optimization
   routine

   Author : Terry Koo
            maestro@csail.mit.edu */

#ifndef EGSTRA_TRAINER_H
#define EGSTRA_TRAINER_H

#include "FGen.h"
#include "StructuredModel.h"
#include "Dataset.h"

#include <string>

#include <assert.h>

namespace egstra {
  class trainer {
  protected:
    /* underlying structured model */
    structured_model* _M;
    /* underlying feature mapping */
    fgen* _fg;
    /* model directory, for parameter and configuration files */
    std::string _dir;

  public:
    trainer() : _M(NULL), _fg(NULL), _dir() {}
    virtual ~trainer() {}

    /* handle the command-line arguments */
    virtual void usage(const char* const mesg = "") = 0;
    virtual void process_options() = 0;

    /* these can be set only once */
    void set_model(structured_model* M) {
      assert(_M == NULL); assert(M != NULL); _M = M;
    }
    void set_fgen(fgen* fg) {
      assert(_fg == NULL); assert(fg != NULL); _fg = fg;
    }
    void set_dir(const std::string& dir) {
      assert(_dir.size() == 0); assert(dir.size() > 0); _dir = dir;
    }

    /* invoke the training routine with the provided model and
       features on the given datasets, training for at most T
       iterations */
    virtual void train(dataset& trdata, dataset& valdata, const int T) = 0;
  };
}

#endif /* EGSTRA_TRAINER_H */
